import React, { useEffect, useState } from "react";
import ReactDOM, {unmountComponentAtNode} from "react-dom";
import Button from "@material-ui/core/Button";

import "dhtmlx-gantt";
import { Gantt } from "dhtmlx-gantt";
import "dhtmlx-gantt/codebase/skins/dhtmlxgantt_broadway.css";
import "./Gantt.css";

const gantt_container = <div className="gantt_here" id="gantt_here"></div>;
let gantt;
function GanttComponent(activeTab) {
  useEffect(() => {
    gantt = Gantt.getGanttInstance();
    console.log("Gantt", Gantt);

    gantt.plugins({
      auto_scheduling: true,
      click_drag: true,
      critical_path: true,
      drag_timeline: true,
      // fullscreen: true,
      grouping: true,
      // keyboard_navigation: true,
      marker: true,
      multiselect: true,
      overlay: true,
      quick_info: true,
      tooltip: true,
      undo: true,
    });

    gantt.config.auto_scheduling = true;
    gantt.config.auto_scheduling_initial = true;

    // custom calendars:
    gantt.config.work_time = true;

    gantt.addCalendar({
      id: "custom",
    });
    gantt.getCalendar("custom").setWorkTime({ date: new Date("2025", "02", "31"), hours: false });
    gantt.getCalendar("custom").setWorkTime({ date: new Date("2025", "03", "01"), hours: false });
    gantt.getCalendar("custom").setWorkTime({ date: new Date("2025", "03", "04"), hours: false });

    gantt.addMarker({
      start_date: new Date("2025", "03", "04"),
      css: "today",
      text: "Today",
    });

    gantt.templates.scale_cell_class = function (date) {
      if (!gantt.isWorkTime(date)) return "weekend";
    };

    gantt.templates.timeline_cell_class = function (item, date) {
      if (!gantt.isWorkTime({ date: date, task: item })) return "weekend";
    };

    gantt.attachEvent("onTaskClick", function (id, e) {
      gantt.message("onTaskClick: " + id);
      return true;
    });

    gantt.config.row_height = 40;
    gantt.config.grid_width = 500;

    gantt.config.columns = [
      { name: "text", label: "Task name", tree: true, width: "*" },
      { name: "start_date", label: "Start time", align: "center" },
      { name: "duration", label: "Duration", align: "center" },
      {
        name: "buttons1",
        label: "Buttons 1",
        align: "center",
        width: 100,
        onrender: (item, node) => {
          return (
            <Button variant="outlined" color="primary" size="small" onClick={() => alert("This is an Outlined Material Button")}>
              outlined
            </Button>
          );
        },
      },
      {
        name: "buttons2",
        label: "Buttons 2",
        align: "center",
        width: 120,
        onrender: (item, node) => {
          return (
            <Button variant="contained" color="secondary" onClick={() => alert("This is a Contained Material Button")}>
              contained
            </Button>
          );
        },
      },
      { name: "add", label: "", width: 44 }
    ];
    gantt.config.external_render = {
      isElement: (element) => {
        return React.isValidElement(element);
      },
      renderElement: (element, container) => {
        ReactDOM.render(element, container);
      },
    };

    function detachReactNodes(){
      var nodes1 = document.querySelectorAll(`.gantt_cell[data-column-index="3"]`)
      nodes1.forEach(function (node) {
        ReactDOM.unmountComponentAtNode(node);
      });
      var nodes2 = document.querySelectorAll(`.gantt_cell[data-column-index="4"]`)
      nodes2.forEach(function (node) {
        ReactDOM.unmountComponentAtNode(node);
      });
    }

    function detachSpecificReactNode(id) {
      ReactDOM.unmountComponentAtNode(document.querySelector(`[data-task-id="${id}"] .gantt_cell[data-column-index="3"]`));
      ReactDOM.unmountComponentAtNode(document.querySelector(`[data-task-id="${id}"] .gantt_cell[data-column-index="4"]`));
    }

    gantt.attachEvent("onBeforeDataRender", function () {
      detachReactNodes();
    });

    gantt.attachEvent("onBeforeTaskSelected", function (id, e) {
      detachSpecificReactNode(id);
      return true;
    });
    gantt.attachEvent("onTaskDrag", function (id, mode, task, original) {
      detachSpecificReactNode(id);
    });

    let linkAdd = null;
    gantt.attachEvent("onMouseMove", function (id, e) {
        if (document.querySelector(".gantt_link_direction") && linkAdd === null) {
            detachSpecificReactNode(id);
            linkAdd = "started";
        }
        if (linkAdd && !document.querySelector(".gantt_link_direction")) {
            linkAdd = null;
        }
    });

    



    // manually detach React elements to prove it is working
    // gantt.attachEvent("onEmptyClick", function (e) {
    //   var nodes = document.querySelectorAll(`.gantt_cell[data-column-index="3"]`)
    //   nodes.forEach(function (node) {
    //     ReactDOM.unmountComponentAtNode(node);
    //   });
    // });


    gantt.config.order_branch = true;
    // restore the order of tasks
    gantt.attachEvent("onLoadEnd", function () {
      gantt.batchUpdate(function () {
        gantt.sort("sort_order", false);
        gantt.sort("sort_order", false);
      });
    });

    gantt.init("gantt_here");
    gantt.load("http://localhost:4500");

    gantt.attachEvent("onRowDragEnd", function (id, target) {
      //update the order of tasks
      gantt.batchUpdate(function () {
        gantt.eachTask(function (task) {
          task.sort_order = task.$index + 1;
          gantt.updateTask(task.id);
        });
      });
    });

    var dp = new gantt.dataProcessor("http://localhost:4500");
    dp.init(gantt);
    dp.setTransactionMode("REST");

    return () => {
      dp.destructor();
      gantt.destructor();
    };
  });

  return gantt_container;
}

export default GanttComponent;
